# Stubs for _imp (Python 3.6)

import types
from typing import Any, List

from importlib.machinery import ModuleSpec
def create_builtin(spec: ModuleSpec) -> types.ModuleType: ...
def create_dynamic(spec: ModuleSpec, file: Any = ...) -> None: ...

def acquire_lock() -> None: ...
def exec_builtin(mod: types.ModuleType) -> int: ...
def exec_dynamic(mod: types.ModuleType) -> int: ...
def extension_suffixes() -> List[str]: ...
def get_frozen_object(name: str) -> types.CodeType: ...
def init_frozen(name: str) -> types.ModuleType: ...
def is_builtin(name: str) -> int: ...
def is_frozen(name: str) -> bool: ...
def is_frozen_package(name: str) -> bool: ...
def lock_held() -> bool: ...
def release_lock() -> None: ...
